@@ -3,6 +3,9 @@
 #if __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Safe #-}
 #endif
+#if __GLASGOW_HASKELL__ >= 706
+{-# LANGUAGE DeriveGeneric #-}
+#endif
 #if __GLASGOW_HASKELL__ >= 710
 {-# LANGUAGE AutoDeriveTypeable #-}
 #endif
@@ -48,6 +51,9 @@ import qualified Data.FingerTree as FT
 import Data.FingerTree (FingerTree, Measured(..), ViewL(..), (<|), (><))
 
 import Prelude hiding (null)
+#if MIN_VERSION_base(4,6,0)
+import GHC.Generics
+#endif
 #if MIN_VERSION_base(4,8,0)
 import qualified Prelude (null)
 #else
@@ -56,7 +62,7 @@ import Data.Foldable (Foldable(foldMap))
 import Data.Monoid
 import Data.Traversable (Traversable(traverse))
 #endif
-#if MIN_VERSION_base(4,9,0)
+#if (MIN_VERSION_base(4,9,0)) && !(MIN_VERSION_base(4,11,0))
 import Data.Semigroup
 #endif
 import Data.Foldable (toList)
@@ -68,7 +74,11 @@ import Data.Foldable (toList)
 -- | A closed interval.  The lower bound should be less than or equal
 -- to the upper bound.
 data Interval v = Interval v v -- ^ Lower and upper bounds of the interval.
-    deriving (Eq, Ord, Show)
+    deriving (Eq, Ord, Show, Read
+#if __GLASGOW_HASKELL__ >= 706
+        , Generic
+#endif
+        )
 
 -- | Lower bound of the interval
 low :: Interval v -> v
@@ -83,7 +93,11 @@ point :: v -> Interval v
 point v = Interval v v
 
 data Node v a = Node (Interval v) a
-    deriving (Eq, Ord, Show)
+    deriving (Eq, Ord, Show, Read
+#if __GLASGOW_HASKELL__ >= 706
+        , Generic
+#endif
+        )
 
 instance Functor (Node v) where
     fmap f (Node i x) = Node i (f x)
@@ -96,6 +110,9 @@ instance Traversable (Node v) where
 
 -- rightmost interval (including largest lower bound) and largest upper bound.
 data IntInterval v = NoInterval | IntInterval (Interval v) v
+#if __GLASGOW_HASKELL__ >= 706
+    deriving (Generic)
+#endif
 
 #if MIN_VERSION_base(4,9,0)
 instance Ord v => Semigroup (IntInterval v) where
@@ -120,6 +137,9 @@ instance (Ord v) => Measured (IntInterval v) (Node v a) where
 -- | Map of closed intervals, possibly with duplicates.
 newtype IntervalMap v a =
     IntervalMap (FingerTree (IntInterval v) (Node v a))
+#if __GLASGOW_HASKELL__ >= 706
+    deriving (Generic)
+#endif
 -- ordered lexicographically by interval
 
 instance Functor (IntervalMap v) where
@@ -236,6 +256,8 @@ inRange lo hi (IntervalMap t) = matches (FT.takeUntil (greater hi) t)
 -- | /O(1)/.  @'bounds' m@ returns @'Nothing'@ if @m@ is empty, and
 -- otherwise @'Just' i@, where @i@ is the smallest interval containing
 -- all the intervals in the map.
+--
+-- @since 0.1.3.0
 bounds :: (Ord v) => IntervalMap v a -> Maybe (Interval v)
 bounds (IntervalMap t) = case measure t of
     NoInterval -> Nothing
@@ -246,6 +268,8 @@ bounds (IntervalMap t) = case measure t of
 -- | /O(1)/.  @'leastView' m@ returns @'Nothing'@ if @m@ is empty, and
 -- otherwise @'Just' ((i, x), m')@, where @i@ is the least interval,
 -- @x@ is the associated value, and @m'@ is the rest of the map.
+--
+-- @since 0.1.3.0
 leastView :: Ord v =>
     IntervalMap v a -> Maybe ((Interval v, a), IntervalMap v a)
 leastView (IntervalMap t) = case FT.viewl t of
@@ -255,6 +279,8 @@ leastView (IntervalMap t) = case FT.viewl t of
 -- | /O(log(min(i,n-i)))/.  @'splitAfter' k m@ returns a pair of submaps,
 -- one consisting of intervals whose lower bound is less than or equal
 -- to @k@, and the other of those whose lower bound is greater.
+--
+-- @since 0.1.3.0
 splitAfter :: Ord v =>
     v -> IntervalMap v a -> (IntervalMap v a, IntervalMap v a)
 splitAfter k (IntervalMap t) = (IntervalMap before, IntervalMap after)
